#include "logical_core.h"
#include <math.h>
LogicalCore::LogicalCore(tf::TransformListener &tf) : ac_("move_base", true), car_state_(IDLE_STATE), tf_(tf)
{
//    blood_record.push_back(1.0);
    go_to_load_bullet = 0;
    blood_rate init_blood;//blood status
    setmoveGoal(bullet_loading_goal_,0,0,1.0,0);//x,y,z,yaw
    init_blood.blood_rate = 1.0;
    init_blood.strike_status = -1;
    init_blood.strike_time = ros::Time::now();
    blood_info.push_back(init_blood);
    
    fillPatrolList();
    ros::NodeHandle nh_cam;
    ros::Rate loop(30);
    //car_pos        = nh_cam.subscribe<geometry_msgs::PoseWithCovarianceStamped>("amcl_pose", 10, boost::bind(&LogicalCore::getCarPos, this, _1));
    send_goal_srv_ = nh_cam.subscribe<move_base_msgs::MoveBaseGoal>("camera_goal", 10, boost::bind(&LogicalCore::getCameraGoal, this, _1));
    sub_current_pos = nh_cam.subscribe<geometry_msgs::PoseWithCovarianceStamped>("amcl_pose", 5, boost::bind(&LogicalCore::pos_callback, this, _1));
    get_strike_ = nh_cam.subscribe("blood_info",1,&LogicalCore::get_strike_callback,this);
    load_bullet_ = nh_cam.subscribe("bullet_info",1,&LogicalCore::load_bullet_callback,this);
    flight_to_send_bullet = nh_cam.advertise<std_msgs::String>("air_bullet_status",5);
    ROS_INFO("Preparation complete to get goals from camera.");
    ROS_INFO("Waiting for action server to start.");
    ac_.waitForServer();
    ROS_INFO("Action server started, ready for sending goals.");

    //goal_.target_pose.pose.x     = 0.0;
    //old_goal_.target_pose.pose.x = 0.0;
    p_point_mode_   = new PointMode(this);
    p_patrol_mode_  = new PatrolMode(this);
    p_shooting_mode_= new ShootingMode(this);
    p_moveback_mode_ = new MoveBackMode(this);
    p_loadbullet_mode_ = new LoadBulletMode(this);
    p_mission_mode_ = p_patrol_mode_;

    old_enemy_global_goal_.pose.position.x = 0;
    old_enemy_global_goal_.pose.position.y = 0;

    
    while(ros::ok())
    {
	//rotateAllist(find_close_pos_in_list(patrol_list_));
	//every time update the Patrollist move the close point to me 
	stateMachine();
        ros::spinOnce();
        loop.sleep();
        
    }
}
void LogicalCore::fillPatrolList()
{
    move_base_msgs::MoveBaseGoal temp_goal;
    temp_goal.target_pose.header.frame_id = "map";

    cv::FileStorage fs("list.yaml", cv::FileStorage::READ);
    if( !fs.isOpened() ) // if we have file with parameters, read them
    {
        std::cout<<"ERROR, cannot open list.yaml!"<<std::endl;
    }
    cv::FileNode list_n = fs["features"];
    cv::FileNodeIterator it = list_n.begin(), it_end = list_n.end();
    for (; it != it_end; ++it)
    {
	setmoveGoal(temp_goal,(double)(*it)["x"],(double)(*it)["y"],1.0,0);
	
	// temp_goal.target_pose.pose.position.x   = (double)(*it)["x"];
	// temp_goal.target_pose.pose.position.y   = (double)(*it)["y"];
	// temp_goal.target_pose.pose.position.z   = 1.;
        std::cout << (double)(*it)["x"] << "  ";
        std::cout << (double)(*it)["y"] << std::endl;
        //temp_goal.target_pose.pose.orientation.w = 1.;
        patrol_list_.push_back(temp_goal);
    }
    std::reverse(patrol_list_.begin(), patrol_list_.end());
    fs.release();
}
void LogicalCore::fillMoveBackList()
{
    move_base_msgs::MoveBaseGoal temp_goal;
    temp_goal.target_pose.header.frame_id = "map";

    cv::FileStorage fs("moveback_list.yaml", cv::FileStorage::READ);
    if( !fs.isOpened() ) // if we have file with parameters, read them
    {
        std::cout<<"ERROR, cannot open list.yaml!"<<std::endl;
    }
    cv::FileNode list_n = fs["features"];
    cv::FileNodeIterator it = list_n.begin(), it_end = list_n.end();
    for (; it != it_end; ++it)
    {
	setmoveGoal(temp_goal,(double)(*it)["x"],(double)(*it)["y"],1.0,0);
      //  temp_goal.target_pose.pose.position.x   = (double)(*it)["x"];
      //  temp_goal.target_pose.pose.position.y   = (double)(*it)["y"];
      //  temp_goal.target_pose.pose.position.z   = 1.;
        std::cout << (double)(*it)["x"] << "  ";
        std::cout << (double)(*it)["y"] << std::endl;
     //  temp_goal.target_pose.pose.orientation.w = 1.;
        MoveBackList_.push_back(temp_goal);
    }
  //  std::reverse(MoveBackList_.begin(), MoveBackList_.end());
    fs.release();
}
geometry_msgs::PoseStamped LogicalCore::goalToGlobalFrame(const geometry_msgs::PoseStamped& goal_pose_msg)
{
    std::string global_frame = "map";
    tf::Stamped<tf::Pose> goal_pose, global_pose;
    poseStampedMsgToTF(goal_pose_msg, goal_pose);

    //just get the latest available transform... for accuracy they should send
    //goals in the frame of the planner
    goal_pose.stamp_ = ros::Time();

    try{
        tf_.transformPose(global_frame, goal_pose, global_pose);
    }
    catch(tf::TransformException& ex){
        ROS_WARN("Failed to transform the goal pose from %s into the %s frame: %s",
                 goal_pose.frame_id_.c_str(), global_frame.c_str(), ex.what());
        return goal_pose_msg;
    }

    geometry_msgs::PoseStamped global_pose_msg;
    tf::poseStampedTFToMsg(global_pose, global_pose_msg);
    return global_pose_msg;
}

double LogicalCore::distance(const geometry_msgs::PoseStamped& p1, const geometry_msgs::PoseStamped& p2)
{
    return hypot(p1.pose.position.x - p2.pose.position.x, p1.pose.position.y - p2.pose.position.y);
}

void LogicalCore::getCameraGoal(const move_base_msgs::MoveBaseGoal::ConstPtr &msg)
{
    if(msg->target_pose.pose.position.z == 1)
    {   
       // ROS_INFO("I want to shooting!");
        enemy_goal_ = msg->target_pose;
        enemy_goal_.pose.position.x -= 2.5;
        /*ROS_INFO("Target position x is %f",enemy_goal_.pose.position.x);  
        ROS_INFO("Target position y is %f",enemy_goal_.pose.position.y); 
        ROS_INFO("Target to shoot or not  is %f",enemy_goal_.pose.position.z);*/
        enemy_global_goal_ = goalToGlobalFrame(enemy_goal_);
        //tf::Pose pose;
        //tf::poseMsgToTF(enemy_global_goal_.pose, pose);
        //double yaw = tf::getYaw(pose.getRotation());
        /*ROS_INFO("current global position x is %f",car_position.pose.position.x);  
        ROS_INFO("current global position y is %f",car_position.pose.position.y); 
        ROS_INFO("current global orientation z is %f",car_position.pose.orientation.z); 
        ROS_INFO("Target global position x is %f",enemy_global_goal_.pose.position.x);  
        ROS_INFO("Target global position y is %f",enemy_global_goal_.pose.position.y); 
        ROS_INFO("Target global to shoot or not  is %f",enemy_global_goal_.pose.position.z);*/
        
        if(distance(enemy_global_goal_, old_enemy_global_goal_) > 1.5)
        { 
	  if(p_mission_mode_ != p_moveback_mode_)
	  {
	     p_mission_mode_ = p_shooting_mode_;
            //goal_.target_pose.pose.orientation.w = 1.0;
            //goal_.target_pose.pose.orientation.z = 0.0;
            car_state_ = OBTAINED_GOAL_STATE;
            old_enemy_global_goal_ = enemy_global_goal_;
            goal_.target_pose = enemy_global_goal_;
            goal_.target_pose.header.stamp = ros::Time::now();
            goal_.target_pose.pose.position.z = 2;
	  }
        }
    }
    else if(msg->target_pose.pose.position.z == 0)
    {
	ROS_INFO("I am looking for you!");
        if(p_mission_mode_ == p_shooting_mode_)
        {
          
            car_state_ = IDLE_STATE;
        }
    }
}

void LogicalCore::pos_callback(const geometry_msgs::PoseWithCovarianceStamped::ConstPtr &pos)
{
    car_position.pose = pos->pose.pose;
    car_position.header = pos->header;
}


void LogicalCore::stateMachine()
{
    switch(car_state_)
    {
    case OBTAINED_GOAL_STATE:
    {
        p_mission_mode_->obtainedGoalState();
        break;
    }
    case BUSY_STATE:
    {
        p_mission_mode_->busyState();
        break;
    }
    case IDLE_STATE:
    {
        p_mission_mode_->idleState();
        break;
    }
    }
}

void LogicalCore::get_strike_callback(const serial_comm::blood_status &blood_status_)
{
  ros::NodeHandle nh;
    
  double blood_lost_rate;
  if(blood_info.back().blood_rate - blood_status_.blood_status >0.02)
  {
    blood_rate current_blood; 
    current_blood.blood_rate= blood_status_.blood_status;
    current_blood.strike_status = blood_status_.strike_status;
    current_blood.strike_time = blood_status_.stamp;
    blood_info.push_back(current_blood);
  }
  vector<blood_rate>::iterator it_end  = blood_info.end();
  vector<blood_rate>::iterator it_ = blood_info.end();
  if(!blood_info.empty())
  {
    if(blood_info.size() > 5)
    {
      it_ = it_ - MAX_GAP_TIME; 
      ros::Duration duration = (it_end->strike_time - it_->strike_time);
      double time_ = duration.toSec();
      blood_lost_rate = (it_->blood_rate - it_end->blood_rate) / time_;
    }
    else
    {
      blood_lost_rate = 0;
    }
  }
 
  if (blood_lost_rate > max_blood_lost_rate)
  {
    ROS_INFO("######################Under Attack#####################");
    moveback_goal  = get_move_back_point(MoveBackList_,it_end->strike_status);

    if(p_mission_mode_ == p_shooting_mode_ && p_mission_mode_ != p_moveback_mode_ )
    {
      if (blood_lost_rate > max_blood_lost_rate * 5)
      {
	ROS_INFO("################Lose too much blood################");
	ROS_INFO("Get move back goal, begin to switch mode to moveback_mode");
	p_mission_mode_ = p_moveback_mode_;
	//goal_.target_pose.pose.orientation.w = 1.0;
	//goal_.target_pose.pose.orientation.z = 0.0;
	car_state_ = OBTAINED_GOAL_STATE;
      }
      else
      {
	 ROS_INFO("Shooting mode ,not too much blood lose, give up move back goal");
	 
      }
    }
    else if (p_mission_mode_ == p_loadbullet_mode_)
    {
      //rotate to find target
    }
  }
  return ;
}

void LogicalCore::load_bullet_callback(const serial_comm::bullet_status &bullet_status_)//bullet_status callback
{  
  if (bullet_status_.bullet_status == 0 && go_to_load_bullet == 0)// have bullet no need for adding bullet!!!!
  {
    std_msgs::String msg;
    std::stringstream ss;
    ss << "1" ;
    msg.data = ss.str();
    flight_to_send_bullet.publish(msg);      //send msg "1", no need to load bullet!!!!
  }
  else if(bullet_status_.bullet_status == 1 && go_to_load_bullet == 0)
    //bullet empty (1), 
    //and not in loading bullet status(go_to_load_bullet == 0)
  {
    if(check_goal_status(bullet_loading_goal_) == true)
    {
        p_mission_mode_ = p_loadbullet_mode_;
	car_state_ = OBTAINED_GOAL_STATE;
	go_to_load_bullet == 1;// on the way to load bullet;
    }
    else
    {
      ROS_INFO("[LOAD_BULLET_MODE] cannot get the position to load bullet!");
    }
  }

  else if (go_to_load_bullet == 1)//flag == 1 on the way to load bullet!!!!!
  {
    std_msgs::String msg;
    std::stringstream ss;
    ss << "2" ;
    msg.data = ss.str();
    flight_to_send_bullet.publish(msg);      //send msg "2", i was coming!!!!
  }
  else if (go_to_load_bullet == 2)//flag == 2 Arrived at the position!!!!!
  {
    ROS_INFO("[LOAD_BULLET_MODE] ARRIVED AT THE POSITION!!! ");
    std_msgs::String msg;
    std::stringstream ss;
    ss << "3" ;
    msg.data = ss.str();
    flight_to_send_bullet.publish(msg);      //send msg "3", i arrived!!!!
  }
  else if(go_to_load_bullet == 3)
  {
    ROS_INFO("[LOAD_BULLET_MODE] FINISHED LOADING BULLET!!! ");
    go_to_load_bullet = 0;
  }
}
/************************************update patrol list****************************************************/
void LogicalCore::setmoveGoal(move_base_msgs::MoveBaseGoal& goal_,double x, double y,double z, double yaw)
{
  goal_.target_pose.pose.position.x = x;
  goal_.target_pose.pose.position.y = y;
  goal_.target_pose.pose.position.z = z;
  goal_.target_pose.pose.orientation.w = 1;
}
bool LogicalCore::check_goal_status(const move_base_msgs::MoveBaseGoal& target_goal)
{
  if(target_goal.target_pose.pose.position.z == 0.0)
  {
    //give global goal to car and navigation to the target
    return false;
  }
  else
  {
    return true;
  }
  
}


void LogicalCore::rotate_patrol_list(int i)
{
  
  move_base_msgs::MoveBaseGoal temp_goal;
  if(patrol_list_.empty())
  {
    ROS_INFO("No target point in patrol_list!,check list.yaml");
    return;
  }
  if(i == -1)
  {
    ROS_INFO("cannot find such point in patrol_list!");
    return;
  }
  while(i != patrol_list_.size())
  {
    temp_goal = patrol_list_.back();
    patrol_list_.pop_back();
    VecMovIt it = patrol_list_.begin();
    patrol_list_.insert(it,temp_goal);
    i++;
  }
  return ;
}
 int LogicalCore::find_close_pos_in_list(const std::vector<move_base_msgs::MoveBaseGoal>& list_)
{
    geometry_msgs::PoseStamped B_pos = car_position;
    double min_distance = 1000000;
    double temp_distance = 0;
    int min_id = -1;
    for (int i = 0 ; i < list_.size(); i++)
    {
	temp_distance = distance(list_[i].target_pose,B_pos);
	if(temp_distance < min_distance)
	{
	  min_distance = temp_distance;
	  min_id = i;
	}
    }
    if(min_distance > 1.0)
    {
      return min_id;
    }
    else
    {
      return -1;
    }
    
}
 int LogicalCore::find_far_pos_in_list(const std::vector<move_base_msgs::MoveBaseGoal>& list_)
{
    geometry_msgs::PoseStamped B_pos = car_position;
    double max_distance = -1;
    double temp_distance = 0;
    int max_id = -1;
    for (int i = 0 ; i < list_.size(); i++)
    {
	temp_distance = distance(list_[i].target_pose,B_pos);
	if(temp_distance > max_distance)
	{
	  max_distance = temp_distance;
	  max_id = i;
	}
    }
    return max_id;
}
bool LogicalCore::find_pose_left_right(const geometry_msgs::PoseStamped& p1, const geometry_msgs::PoseStamped& p2,const geometry_msgs::PoseStamped& p3)
{
  double triangle_area = ((p1.pose.position.x - p3.pose.position.x)*(p2.pose.position.y - p3.pose.position.y) \
  - (p1.pose.position.y - p3.pose.position.y)*(p2.pose.position.x - p3.pose.position.x))/2;
  return (triangle_area > 0 ? true : false);//right is false ###### left is true;
}
////////////////////////////////////////////////////////////////////////////////////////////////////////////
move_base_msgs::MoveBaseGoal LogicalCore::get_move_back_point(const std::vector<move_base_msgs::MoveBaseGoal> &movebacklist,int boardDirection)
{
  //
    geometry_msgs::PoseStamped temp_goal;
    geometry_msgs::PoseStamped global_temp_goal;
    vector<move_base_msgs::MoveBaseGoal> temp_list;
    geometry_msgs::PoseStamped B_pos = car_position;
    geometry_msgs::PoseStamped A_pos ;
    switch (boardDirection)
    {
      case 0://front
      {
	temp_goal.pose.position.x = 0;
	temp_goal.pose.position.y = 1;
	temp_goal.pose.position.z = 1;      
	break;
      }
      case 1://left
      {
	temp_goal.pose.position.x = -1;
	temp_goal.pose.position.y = 0;
	temp_goal.pose.position.z = 1; 
	break;
      }
      case 2://back
      {
	temp_goal.pose.position.x = 0;
	temp_goal.pose.position.y = -1;
	temp_goal.pose.position.z = 1; 
	break;
      }
      case 3://right
      {
	temp_goal.pose.position.x = 1;
	temp_goal.pose.position.y = 0;
	temp_goal.pose.position.z = 1; 
	break;
      }
      case -1:
	//no hurt!!!!!
	break;
      default:
	break;
    }
    global_temp_goal = goalToGlobalFrame(temp_goal);
    A_pos = global_temp_goal;
    if(!MoveBackList_.empty())
    {
      for (int i = 0 ; i < MoveBackList_.size();i++)
      {
	  if(false  == 	find_pose_left_right(A_pos,B_pos,MoveBackList_[i].target_pose))
	  {
	    temp_list.push_back(MoveBackList_[i]);
	  }
      }
      int p3 = find_close_pos_in_list(temp_list);
      if(p3 == -1)//no such point
      {
	//go to the farest point
	  p3 = find_far_pos_in_list(temp_list);
	  ROS_INFO("[CAL_MOVEBACK_POINT] Finished ,choose far point! : %f , %f ",temp_list[p3].target_pose.pose.position.x,temp_list[p3].target_pose.pose.position.y);
	  return temp_list[p3];
      }
      else
      {
	  ROS_INFO("[CAL_MOVEBACK_POINT] Finished ,choose close point! : %f , %f ",temp_list[p3].target_pose.pose.position.x,temp_list[p3].target_pose.pose.position.y);
	  return temp_list[p3];
      }
    }
    else
    {
      ROS_INFO("[CAL_MOVEBACK_POINT] MoveBackList is empty, check moveback_list.yaml!");
    }
}