#ifndef LOGICALCORE_H
#define LOGICALCORE_H
#include <serial_comm/blood_status.h>
#include <serial_comm/bullet_status.h>
#include <move_base_msgs/MoveBaseAction.h>
#include <geometry_msgs/PoseWithCovarianceStamped.h>
#include <actionlib/client/simple_action_client.h>
#include <std_msgs/String.h>
#include <vector>
#include <opencv2/opencv.hpp>
#include <string>
#include <algorithm>
#include "mission_mode.h"
#include "point_mode.h"
#include "patrol_mode.h"
#include "shooting_mode.h"
#include "loadbullet_mode.h"
#include "moveback_mode.h"
#include <ros/ros.h>
#include <tf/tf.h>
#include <tf/transform_listener.h>
using namespace std;
typedef actionlib::SimpleActionClient<move_base_msgs::MoveBaseAction> MoveBaseClient;
typedef std::vector<move_base_msgs::MoveBaseGoal>::iterator VecMovIt;

const double max_blood_lost_rate = 0.3;
const int MAX_GAP_TIME = 10;
enum CarState
{
  OBTAINED_GOAL_STATE,
  BUSY_STATE,
  IDLE_STATE
};
typedef struct blood_rate_
{
  double blood_rate;
  ros::Time strike_time;
  int strike_status;
} blood_rate;
class LogicalCore
{
public:
  friend class PointMode;
  friend class PatrolMode;
  friend class ShootingMode;
  friend class LoadBulletMode;
  friend class MoveBackMode;
  
  int go_to_load_bullet;//0 for no 1 for ing 2 for finished
   
  LogicalCore(tf::TransformListener &tf);
  void get_strike_callback(const serial_comm::blood_status &blood_status_);//blood info callback
  void load_bullet_callback(const serial_comm::bullet_status &bullet_status_);//bullet_status callback
  
  move_base_msgs::MoveBaseGoal get_move_back_point(const std::vector<move_base_msgs::MoveBaseGoal> &movebacklist,int boardDirection);
  void pos_callback(const geometry_msgs::PoseWithCovarianceStamped::ConstPtr &pos);
  
  void getCarPos(const geometry_msgs::PoseWithCovarianceStamped::ConstPtr &pos);
  /**
   * @brief The callback function of the camera_goal publishing by armor_detect node.
   * @param msg
   */
  void getCameraGoal(const move_base_msgs::MoveBaseGoal::ConstPtr &msg);//camera callback function
  /**
   * @brief You can set some points in file ./param_manager/autocar_params.xml first,
   * this function will load the parameters to the vector patrol_list_.
   */ 
  void setmoveGoal(move_base_msgs::MoveBaseGoal& goal_,double x, double y,double z, double yaw);
  bool check_goal_status(const move_base_msgs::MoveBaseGoal& target_goal);
  void fillPatrolList();
  void fillMoveBackList();
  void rotate_patrol_list(int i);
  int find_close_pos_in_list(const std::vector<move_base_msgs::MoveBaseGoal>& list_);
  int find_far_pos_in_list(const std::vector<move_base_msgs::MoveBaseGoal>& list_);
  bool find_pose_left_right(const geometry_msgs::PoseStamped& p1, const geometry_msgs::PoseStamped& p2,const geometry_msgs::PoseStamped& p3);
  double distance(const geometry_msgs::PoseStamped& p1, const geometry_msgs::PoseStamped& p2);
  geometry_msgs::PoseStamped goalToGlobalFrame(const geometry_msgs::PoseStamped &goal_pose_msg);
  /**
   * @brief stateMachine the logical core of autocar.
   */
  void stateMachine();
private:
  //We should init the patrol_list first.
  std::vector<move_base_msgs::MoveBaseGoal> patrol_list_;
  move_base_msgs::MoveBaseGoal old_patrol_goal_;
  move_base_msgs::MoveBaseGoal curr_patrol_goal_;
   /****************************************************/
  //std::vector<move_base_msgs::MoveBaseGoal> patrol_list_;
  std::vector<move_base_msgs::MoveBaseGoal> MoveBackList_;//move back list
  move_base_msgs::MoveBaseGoal moveback_goal;
  move_base_msgs::MoveBaseGoal bullet_loading_goal_;//goal for loading bullet
  
 // vector<double> get_strkie_time;//blood_infomation
  vector<blood_rate> blood_info;//history blood information
  vector<double> blood_record;
   /****************************************************/
  MoveBaseClient ac_;
  tf::TransformListener &tf_;
  move_base_msgs::MoveBaseGoal goal_;
  geometry_msgs::PoseStamped old_enemy_global_goal_;
  geometry_msgs::PoseStamped enemy_goal_;
  geometry_msgs::PoseStamped enemy_global_goal_;
  geometry_msgs::PoseStamped car_position;
  ros::Subscriber send_goal_srv_;
  ros::Subscriber sub_current_pos;
  ros::Publisher flight_to_send_bullet;
  /****************************************************/
  ros::Subscriber get_strike_;//if get_strike 
  ros::Subscriber load_bullet_;// go to load bullet
  ros::Subscriber bullet_full_;
 
  /****************************************************/
  CarState car_state_;
  MissionMode  *p_mission_mode_;
  PointMode    *p_point_mode_;
  PatrolMode   *p_patrol_mode_;
  ShootingMode *p_shooting_mode_;
  /****************************************************/
  LoadBulletMode *p_loadbullet_mode_;
  MoveBackMode *p_moveback_mode_;
  
};

#endif // LOGICALCORE_H
