#include <ros/ros.h>
#include <geometry_msgs/Vector3.h>
#include <sensor_msgs/Image.h>
#include <opencv2/opencv.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include "opencv2/highgui/highgui.hpp"
#include <cv_bridge/cv_bridge.h>
#include <image_transport/image_transport.h>
#include <algorithm>
using namespace cv;
using namespace std;


int thresh = 100;
int max_thresh = 255;
RNG rng(12345);

#include <iostream>
#include <stdlib.h>
#ifndef SHOW_DEBUG_IMG
#define SHOW_DEBUG_IMG
#endif
#ifndef DEBUG_CORE
//#define DEBUG_CORE
#endif
bool rect_compare(const RotatedRect& r1,const RotatedRect& r2)
{
  return r1.size.width >r2.size.width;
}
class circleRANSAC {
public:
  circleRANSAC(ros::NodeHandle& nh,ros::NodeHandle& pnh);
  bool RANSAC(Mat &image, std::vector<Vec3f> &circles, double canny_threshold, double circle_threshold,int min_radius_in,int numIterations);
  void CirclesDetectorCb(const sensor_msgs::ImageConstPtr& msg,const sensor_msgs::CameraInfoConstPtr& cam_info);
private:

  Mat src;
  image_transport::ImageTransport it_;
  image_transport::CameraSubscriber image_sub_;
  geometry_msgs::Vector3 circle_pose;
  ros::Publisher  pose_pub_;// = nh.advertise<geometry_msgs::Vector3>("circle_relative_pose", 1);
};


int low_r ,low_g,low_b;
int high_r ,high_g,high_b;

void on_low_r_thresh_trackbar(int, void *);
void on_high_r_thresh_trackbar(int, void *);
void on_low_g_thresh_trackbar(int, void *);
void on_high_g_thresh_trackbar(int, void *);
void on_low_b_thresh_trackbar(int, void *);
void on_high_b_thresh_trackbar(int, void *);

void on_low_r_thresh_trackbar(int, void *)
{
    low_r = min(high_r-1, low_r);
    setTrackbarPos("Low R","Object Detection", low_r);
}
void on_high_r_thresh_trackbar(int, void *)
{
    high_r = max(high_r, low_r+1);
    setTrackbarPos("High R", "Object Detection", high_r);
}
void on_low_g_thresh_trackbar(int, void *)
{
    low_g = min(high_g-1, low_g);
    setTrackbarPos("Low G","Object Detection", low_g);
}
void on_high_g_thresh_trackbar(int, void *)
{
    high_g = max(high_g, low_g+1);
    setTrackbarPos("High G", "Object Detection", high_g);
}
void on_low_b_thresh_trackbar(int, void *)
{
    low_b= min(high_b-1, low_b);
    setTrackbarPos("Low B","Object Detection", low_b);
}
void on_high_b_thresh_trackbar(int, void *)
{
    high_b = max(high_b, low_b+1);
    setTrackbarPos("High B", "Object Detection", high_b);
}
float verifyCircle(cv::Mat& dt, cv::Point2f center, float radius, std::vector<cv::Point2f> & inlierSet)
{
 unsigned int counter = 0;
 unsigned int inlier = 0;
 float minInlierDist = 80.0f;
 float maxInlierDistMax = 120.0f;
 float maxInlierDist = radius/25.0f;
 if(maxInlierDist<minInlierDist) maxInlierDist = minInlierDist;
 if(maxInlierDist>maxInlierDistMax) maxInlierDist = maxInlierDistMax;

 // choose samples along the circle and count inlier percentage
 for(float t =0; t<2*3.14159265359f; t+= 0.05f)
 {
     counter++;
     float cX = radius*cos(t) + center.x;
     float cY = radius*sin(t) + center.y;

     if(cX < dt.cols)
     if(cX >= 0)
     if(cY < dt.rows)
     if(cY >= 0)
     if(dt.at<float>(cY,cX) < maxInlierDist)
     {
        inlier++;
        inlierSet.push_back(cv::Point2f(cX,cY));
     }
 }

 return (float)inlier/float(counter);
}


inline void getCircle(cv::Point& p1,cv::Point& p2,cv::Point& p3, cv::Point2f& center, float& radius)
{
  float x1 = p1.x;
  float x2 = p2.x;
  float x3 = p3.x;

  float y1 = p1.y;
  float y2 = p2.y;
  float y3 = p3.y;

  // PLEASE CHECK FOR TYPOS IN THE FORMULA :)
  center.x = (x1*x1+y1*y1)*(y2-y3) + (x2*x2+y2*y2)*(y3-y1) + (x3*x3+y3*y3)*(y1-y2);
  center.x /= ( 2*(x1*(y2-y3) - y1*(x2-x3) + x2*y3 - x3*y2) );

  center.y = (x1*x1 + y1*y1)*(x3-x2) + (x2*x2+y2*y2)*(x1-x3) + (x3*x3 + y3*y3)*(x2-x1);
  center.y /= ( 2*(x1*(y2-y3) - y1*(x2-x3) + x2*y3 - x3*y2) );

  radius = sqrt((center.x-x1)*(center.x-x1) + (center.y-y1)*(center.y-y1));
}

std::vector<cv::Point2f> getPointPositions(cv::Mat binaryImage)
{
 std::vector<cv::Point2f> pointPositions;

 for(unsigned int y=0; y<binaryImage.rows; ++y)
 {
     //unsigned char* rowPtr = binaryImage.ptr<unsigned char>(y);
     for(unsigned int x=0; x<binaryImage.cols; ++x)
     {
         //if(rowPtr[x] > 0) pointPositions.push_back(cv::Point2i(x,y));
         if(binaryImage.at<unsigned char>(y,x) > 0) pointPositions.push_back(cv::Point2f(x,y));
     }
 }

 return pointPositions;
}

circleRANSAC::circleRANSAC(ros::NodeHandle& nh,ros::NodeHandle& pnh):it_(nh)
{
  image_sub_ = it_.subscribeCamera("/usb_cam/image_raw", 1, &circleRANSAC::CirclesDetectorCb, this);
  pose_pub_ = nh.advertise<geometry_msgs::Vector3>("circle_relative_pose", 1);
}
bool circleRANSAC::RANSAC(Mat &image, std::vector<Vec3f> &circles, double canny_threshold, double circle_threshold,int min_radius_in,int numIterations)
{
#ifdef DEBUG_CORE
    cout<<"debug 1.1"<<endl;
#endif
	//cout<<"S"<<endl;
	CV_Assert(image.type() == CV_8UC1 || image.type() == CV_8UC3);
	//cout<<"B"<<endl;
	if(!circles.empty())
	{
	    circles.clear();
	}
#ifdef DEBUG_CORE
    cout<<"debug 1.2"<<endl;
#endif
	// Edge Detection
	Mat edges;
	Canny(image, edges, MAX(canny_threshold/2,1), canny_threshold, 3);
	if(edges.empty())
	{
	   return false;
	}
#ifdef DEBUG_CORE
    cout<<"debug 1.3"<<endl;
#endif
	//cout<<"D"<<endl;
#ifdef SHOW_DEBUG_IMG
	imshow("Edges",edges);
	waitKey(1);
#endif
	
	//cout<<"J"<<endl;
	// Create point set from Canny Output
	std::vector<Point2d> points;

	for(int r = 0; r < edges.rows; r++)
	{
		for(int c = 0; c < edges.cols; c++)
		{
			if(edges.at<unsigned char>(r,c) == 255)
			{
				points.push_back(cv::Point2d(c,r));
			}
		}	
	}
#ifdef DEBUG_CORE
    cout<<"debug 1.4"<<endl;
#endif
	// 4 point objects to hold the random samples
	Point2d pointA;
	Point2d pointB;
	Point2d pointC;
	Point2d pointD;
	
	// distances between points
	double AB;
	double BC;
	double CA;
	double DC;

	// varibales for line equations y = mx + b
	double m_AB;
	double b_AB;
	double m_BC;
	double b_BC;

	// varibles for line midpoints
	double XmidPoint_AB;
	double YmidPoint_AB;
	double XmidPoint_BC;
	double YmidPoint_BC;

	// variables for perpendicular bisectors
	double m2_AB;
	double m2_BC;
	double b2_AB;
	double b2_BC;

	// RANSAC
	cv::RNG rng; 
	int min_point_separation = 20; // change to be relative to image size?
	int colinear_tolerance = 1; // make sure points are not on a line
	int radius_tolerance = 5; // change to be relative to image size?
	int points_threshold = 10; //should always be greater than 4
	//double min_circle_separation = 10; //reject a circle if it is too close to a previously found circle
	double min_radius = min_radius_in; //minimum radius for a circle to not be rejected
	
	int x,y;
	Point2d center;
	double radius;
#ifdef DEBUG_CORE
    cout<<"debug 1.5"<<endl;
#endif
	// Iterate
	if(!points.empty())
	{
	 for(int iteration = 0; iteration < numIterations; iteration++) 
	  {
		//std::cout << "RANSAC iteration: " << iteration << std::endl;
		
		// get 4 random points
		pointA = points[rng.uniform((int)0, (int)points.size())];
		pointB = points[rng.uniform((int)0, (int)points.size())];
		pointC = points[rng.uniform((int)0, (int)points.size())];
		pointD = points[rng.uniform((int)0, (int)points.size())];
#ifdef DEBUG_CORE
    cout<<"debug 1.5.-1.0"<<endl;
#endif	
		// calc lines
		AB = norm(pointA - pointB);
		BC = norm(pointB - pointC);
		CA = norm(pointC - pointA);
		DC = norm(pointD - pointC);
#ifdef DEBUG_CORE
    cout<<"debug 1.5.-1.1"<<endl;
#endif	
		// one or more random points are too close together
		if(AB < min_point_separation || BC < min_point_separation || CA < min_point_separation || DC < min_point_separation) continue;
#ifdef DEBUG_CORE
    cout<<"debug 1.5.-1.2"<<endl;
#endif		
		//find line equations for AB and BC
		//AB
		m_AB = (pointB.y - pointA.y) / (pointB.x - pointA.x + 0.000000001); //avoid divide by 0
		b_AB = pointB.y - m_AB*pointB.x;

		//BC
		m_BC = (pointC.y - pointB.y) / (pointC.x - pointB.x + 0.000000001); //avoid divide by 0
		b_BC = pointC.y - m_BC*pointC.x;
		
		
		//test colinearity (ie the points are not all on the same line)
		if(abs(pointC.y - (m_AB*pointC.x + b_AB + colinear_tolerance)) < colinear_tolerance) continue;
#ifdef DEBUG_CORE
    cout<<"debug 1.5.-1.3"<<endl;
#endif	
		//find perpendicular bisector
		//AB
		//midpoint
		XmidPoint_AB = (pointB.x + pointA.x) / 2.0;
		YmidPoint_AB = m_AB * XmidPoint_AB + b_AB;
		//perpendicular slope
		m2_AB = -1.0 / m_AB;
		//find b2
		b2_AB = YmidPoint_AB - m2_AB*XmidPoint_AB;

		//BC
		//midpoint
		XmidPoint_BC = (pointC.x + pointB.x) / 2.0;
		YmidPoint_BC = m_BC * XmidPoint_BC + b_BC;
		//perpendicular slope
		m2_BC = -1.0 / m_BC;
		//find b2
		b2_BC = YmidPoint_BC - m2_BC*XmidPoint_BC;
		
		//find intersection = circle center
		x = (b2_AB - b2_BC) / (m2_BC - m2_AB);
		y = m2_AB * x + b2_AB;	
		center = Point2d(x,y);
		radius = cv::norm(center - pointB);
#ifdef DEBUG_CORE
    cout<<"debug 1.5.0.1"<<endl;
#endif
		/// geometry debug image
		if(false)
		{
			Mat debug_image = edges.clone();
			cvtColor(debug_image, debug_image, CV_GRAY2RGB);
		
			Scalar pink(255,0,255);
			Scalar blue(255,0,0);
			Scalar green(0,255,0);
			Scalar yellow(0,255,255);
			Scalar red(0,0,255);
		
			// the 3 points from which the circle is calculated in pink
			circle(debug_image, pointA, 3, pink);
			circle(debug_image, pointB, 3, pink);
			circle(debug_image, pointC, 3, pink);
		
			// the 2 lines (blue) and the perpendicular bisectors (green)
			line(debug_image,pointA,pointB,blue);
			line(debug_image,pointB,pointC,blue);
			line(debug_image,Point(XmidPoint_AB,YmidPoint_AB),center,green);
			line(debug_image,Point(XmidPoint_BC,YmidPoint_BC),center,green);
		
			circle(debug_image, center, 3, yellow); // center
			circle(debug_image, center, radius, yellow);// circle
		
			// 4th point check
			circle(debug_image, pointD, 3, red);
		
			imshow("ransac debug", debug_image);
			waitKey(1);
		}
		
		//check if the 4 point is on the circle
#ifdef DEBUG_CORE
    cout<<"debug 1.5.0.2"<<endl;
#endif
		if(abs(cv::norm(pointD - center) - radius) > radius_tolerance) continue;
#ifdef DEBUG_CORE
    cout<<"debug 1.5.1"<<endl;
#endif
		// vote
		std::vector<int> votes;
		std::vector<int> no_votes;
		
		  for(int i = 0; i < (int)points.size(); i++) 
		  {
			double vote_radius = norm(points[i] - center);
			
			if(abs(vote_radius - radius) < radius_tolerance) 
			{
				votes.push_back(i);
			}
			else
			{
				no_votes.push_back(i);
			}
		  }
#ifdef DEBUG_CORE
    cout<<"debug 1.5.2"<<endl;
#endif

#ifdef DEBUG_CORE
    cout<<"debug 1.5.3"<<endl;
#endif
		// check votes vs circle_threshold
		if(!votes.empty())
		{
		  if( (float)votes.size() / (2.0*CV_PI*radius) >= circle_threshold &&radius >min_radius)
		{
			circles.push_back(Vec3f(x,y,radius));
			
			// voting debug image
			if(false)
			{
				Mat debug_image2 = edges.clone();
				cvtColor(debug_image2, debug_image2, CV_GRAY2RGB);
		
				Scalar yellow(0,255,255);
				Scalar green(0,255,0);
			
				circle(debug_image2, center, 3, yellow); // center
				circle(debug_image2, center, radius, yellow);// circle
			
				// draw points that voted
				for(int i = 0; i < (int)votes.size(); i++)
				{
					circle(debug_image2, points[votes[i]], 1, green);
				}
			
				imshow("ransac debug", debug_image2);
				waitKey(1);
			}
			
			// remove points from the set so they can't vote on multiple circles
			std::vector<Point2d> new_points;
			for(int i = 0; i < (int)no_votes.size(); i++)
			{
				new_points.push_back(points[no_votes[i]]);
			}
			points.clear();
			points = new_points;		
		}
		
		}
		

#ifdef DEBUG_CORE
    cout<<"debug 1.5.4"<<endl;
#endif
		// stop RANSAC if there are few points left
		
		if((int)points.size() < points_threshold )
			break;
#ifdef DEBUG_CORE
    cout<<"debug 1.6"<<endl;
#endif
	  }

 
	}
	else
	{
	  return false;
	}
#ifdef DEBUG_CORE
    cout<<"debug 1.7"<<endl;
#endif
	return true;
}
void circleRANSAC::CirclesDetectorCb(const sensor_msgs::ImageConstPtr& msg,const sensor_msgs::CameraInfoConstPtr& cam_info)
{
#ifdef DEBUG_CORE
    cout<<"debug 1"<<endl;
#endif
    cv_bridge::CvImagePtr cv_ptr;
    try{
	cv_ptr = cv_bridge::toCvCopy(msg, sensor_msgs::image_encodings::BGR8);
        src=cv_ptr->image.clone(); //load  image
#ifdef DEBUG_CORE
    cout<<"debug 2"<<endl;
#endif
    }
    catch (cv_bridge::Exception& e){
	ROS_ERROR("cv_bridge exception: %s", e.what());
	return;
    }
#ifdef DEBUG_CORE
    cout<<"debug 3"<<endl;
#endif
    Mat hsv;
    Mat gray;
    Mat canny_output;
    GaussianBlur( src, src, Size(3, 3), 1.5, 1.5 );
    cvtColor(src, hsv, CV_BGR2HSV);// hsv space transform
   // inRange(hsv,Scalar(0,160,30), Scalar(200,255,255),gray);
   //   inRange(hsv,Scalar(16,0,104), Scalar(157,140,240),gray);
     inRange(hsv,Scalar(98,11,49), Scalar(255,243,196),gray);
#ifdef DEBUG_CORE
    cout<<"debug 4"<<endl;
#endif
#ifdef SHOW_DEBUG_IMG
    imshow("red channel",gray);
    waitKey(1);
#endif
    //GaussianBlur( gray, gray, Size(3, 3), 1.5, 1.5 );
    //adaptiveThreshold(gray, gray, 255, CV_ADAPTIVE_THRESH_MEAN_C, CV_THRESH_BINARY,55, 7);
    double min, max;
    cv::minMaxLoc(gray,&min,&max);
    float sub = min;
    float mult = 255.0f/(float)(max-sub);
    cv::Mat normalized = gray - sub;
    normalized = mult * normalized;
    Canny( normalized, canny_output, thresh, thresh*2, 3 );
    vector<vector<Point> > contours;
    vector<Vec4i> hierarchy;
    vector<RotatedRect> vec_rect;
    int max_area = 400;
    findContours( canny_output, contours, hierarchy, CV_RETR_TREE, CV_CHAIN_APPROX_SIMPLE, Point(0, 0) );
   // Mat cimages = Mat::zeros(canny_output.size(),CV_8UC3);
    if(contours.empty())
      return ;
    for(size_t i = 0; i < contours.size();i++)
    {
      size_t count = contours[i].size();
      double area = fabs(contourArea(contours[i]));
      if(area < max_area|| count < 10 )
	continue;
      Mat pointsf;
      Mat(contours[i]).convertTo(pointsf,CV_32F);
      RotatedRect box = fitEllipse(pointsf);
      
      if(abs(box.size.width - box.size.height )> 10  )
	continue;
      
      //drawContours(src, contours, (int)i, Scalar::all(255), 1, 8);
      //ellipse(src, box, Scalar(0,0,255), 1, LINE_AA);
      //ellipse(src, box.center, box.size*0.5f, box.angle, 0, 360, Scalar(0,255,255), 1, LINE_AA);
      //Point2f vtx[4];
      //box.points(vtx);
      //for( int j = 0; j < 4; j++ )
       //      line(src, vtx[j], vtx[(j+1)%4], Scalar(0,255,0), 1, LINE_AA);
     //}
     // imshow("result", src);
     // cvWaitKey(1);
      vec_rect.push_back(box);
     }
     sort(vec_rect.begin(),vec_rect.end(),rect_compare);
     
/*
   // int erosion_size = 0.5;
   // Mat element = getStructuringElement(MORPH_RECT, Size(15, 15));  
   // Mat element = getStructuringElement( MORPH_RECT,
   //                                   Size( 2*erosion_size + 1, 2*erosion_size+1 ),
   //                                   Point( erosion_size, erosion_size ) );
   // 进行膨胀操作  
   // dilate(normalized,normalized, element);
   // imshow("NORMALIZED_THRESH_BINARY", normalized);  
   // waitKey(1);
   // threshold(normalized, mask, 30, 255, THRESH_BINARY);
 //   int erosion_size = 1;
    //Mat element = getStructuringElement(MORPH_RECT, Size(15, 15));  
  //  Mat element = getStructuringElement( MORPH_RECT,
  //                                     Size( 2*erosion_size + 1, 2*erosion_size+1 ),
  //                                     Point( erosion_size, erosion_size ) );
    //进行膨胀操作  
 //   dilate(normalized,normalized, element);

  //  imshow("NORMALIZED_THRESH_BINARY", normalized);  
 //   waitKey(1);
/*
    cv::Mat mask;
    Mat element = getStructuringElement(MORPH_RECT, Size(2 * 2 + 1,
                    2 * 2 + 1));
    morphologyEx(normalized, mask, MORPH_OPEN, element);
        
    imshow("NORMALIZED_THRESH_BINARY", mask);  
    waitKey(1);
*/
/*********************************************************************/
/*
    double canny_threshold = 180;
    double circle_threshold = 0.80;
    int iterations  = 800;
    vector<Vec3f> circles;
    int x = 0;
    int y = 0;
    float rad;
#ifdef DEBUG_CORE
    cout<<"debug 5"<<endl;
#endif
    bool res = RANSAC(normalized, circles, canny_threshold, circle_threshold,50,iterations);
#ifdef DEBUG_CORE
    cout<<"debug 6"<<endl;
#endif
    if(!circles.empty()&&res)
    {
     for(int i = 0; i < (int)circles.size(); i++)
      {
	 x += circles[i][0];
	 y += circles[i][1];
	  rad = circles[i][2];
	 circle_pose.x = circles[i][0];
         circle_pose.y = circles[i][1];
         circle_pose.z = circles[i][2];
         pose_pub_.publish(circle_pose);
      }
	x = x/circles.size();
	y = y/circles.size();
	ROS_INFO("circle center : %d,%d",x,y);
	circle(src, Point(x,y), rad, Scalar(0,255,0),3);
    }
#ifdef DEBUG_CORE
    cout<<"debug 7"<<endl;
#endif
#ifdef SHOW_DEBUG_IMG
    imshow("circles", src);
    waitKey(1);
#endif

//    std::vector<cv::Point2f> edgePositions;
//  edgePositions = getPointPositions(mask);
    
    Mat edge;
    cv::Canny(normalized, edge, 160, 80);
    std::vector<cv::Point> edge_Points;
    cv::findNonZero(edge, edge_Points);
    cv::Mat dt;
    imshow("Edge map",edge);
   // HoughCircles();
    
    
    cv::distanceTransform(255-edge, dt,CV_DIST_L1, 3);

    unsigned int nIterations = 0;

    cv::Point2f bestCircleCenter;
    float bestCircleRadius;
    float bestCirclePercentage = 0;
    float minRadius = 50	;   // TODO: ADJUST THIS PARAMETER TO YOUR NEEDS, 
    
    //otherwise smaller circles wont be detected or "small noise circles" will have a high percentage of completion
    float minCirclePercentage = 0.60f;  // at least 5% of a circle must be present? maybe more...

    int maxNrOfIterations = edge_Points.size();   // TODO: adjust this parameter or include some real
    //ransac criteria with inlier/outlier percentages to decide when to stop
    
    cv::RNG rng(1234);
    
    for(unsigned int its=0; its< maxNrOfIterations; ++its)
    {
        //RANSAC: randomly choose 3 point and create a circle:
        //TODO: choose randomly but more intelligent, 
        //so that it is more likely to choose three points of a circle. 
        //For example if there are many small circles, it is unlikely to randomly choose 3 points of the same circle.
         unsigned int idx1 = rng.uniform(0, (int)edge_Points.size());
        unsigned int idx2 = rng.uniform(0, (int)edge_Points.size());
        unsigned int idx3 = rng.uniform(0, (int)edge_Points.size());

        // we need 3 different samples:
        if(idx1 == idx2) continue;
        if(idx1 == idx3) continue;
        if(idx3 == idx2) continue;

        // create circle from 3 points:
        cv::Point2f center; 
	float radius;
        getCircle(edge_Points[idx1],edge_Points[idx2],edge_Points[idx3],center,radius);

        // inlier set unused at the moment but could be used to approximate a (more robust) circle from alle inlier
        std::vector<cv::Point2f> inlierSet;

        //verify or falsify the circle by inlier counting:
        float cPerc = verifyCircle(dt,center,radius, inlierSet);

        // update best circle information if necessary
        if(cPerc >= bestCirclePercentage)
	{
	    if(radius >= minRadius)
	    {
	      bestCirclePercentage = cPerc;
	      bestCircleRadius = radius;
	      bestCircleCenter = center;
	    }
	}
            

    }

    // draw if good circle was found
    if(bestCirclePercentage >= minCirclePercentage)
    {
      if(bestCircleRadius >= minRadius)
      {
	cv::circle(src, bestCircleCenter,bestCircleRadius, cv::Scalar(255,0,0),3);
	cv::circle(src, bestCircleCenter,1,cv::Scalar(0,255,0),5);
        cv::imshow("output",src);
        //cv::imshow("mask",mask);
        cv::waitKey(1);
      }
      
    }


*/  if(!vec_rect.empty())
    {
      Point2f vtx[4];
    vec_rect[0].points(vtx);
    ellipse(src, vec_rect[0], Scalar(0,255,0), 3, LINE_AA);
    for( int j = 0; j < 4; j++ )
    {
	line(src, vtx[j], vtx[(j+1)%4], Scalar(0,255,0), 3, LINE_AA);
    }
    }
    
    imshow("result", src);
    cvWaitKey(1);
}

int main(int argc, char **argv){
  /*	
    ros::NodeHandle nh;
    cout<<"init"<<endl;
    image_transport::ImageTransport it(nh);
    cout<<"image_transport"<<endl;
    image_transport::Subscriber image_sub_ = it.subscribe("/usb_cam/image_raw", 1, CirclesDetectorCb);
    pose_pub_ = nh.advertise<geometry_msgs::Vector3>("circle_relative_pose", 1);
    //ROS_INFO("heheheheh!");
    ros::Rate loop_rate(10);
    ros::spin();
    */
  
   ros::init(argc, argv, "circle_detection_node");
   cout<<"1"<<endl;
   ros::NodeHandle nh;
   ros::NodeHandle pnh("~");
   cout<<"2"<<endl;
   circleRANSAC circle_detector(nh,pnh);
   cout<<"3"<<endl;
   ros::spin();
   return 0;
  
 /*
 low_r = low_g = low_b = 0;
high_r = high_g = high_b = 255;
    Mat frame, frame_threshold;
    Mat hsv;
     
    VideoCapture cap(0);
    namedWindow("Video Capture", WINDOW_NORMAL);
    namedWindow("Object Detection", WINDOW_NORMAL);
    //-- Trackbars to set thresholds for RGB values
    createTrackbar("Low H","Object Detection", &low_r, 255, on_low_r_thresh_trackbar);
    createTrackbar("High H","Object Detection", &high_r, 255, on_high_r_thresh_trackbar);
    createTrackbar("Low S","Object Detection", &low_g, 255, on_low_g_thresh_trackbar);
    createTrackbar("High S","Object Detection", &high_g, 255, on_high_g_thresh_trackbar);
    createTrackbar("Low V","Object Detection", &low_b, 255, on_low_b_thresh_trackbar);
    createTrackbar("High V","Object Detection", &high_b, 255, on_high_b_thresh_trackbar);
    while((char)waitKey(1)!='q'){
        cap>>frame;
	cvtColor(frame, hsv, CV_BGR2HSV);// hsv space transform
        if(hsv.empty())
            break;
        //-- Detect the object based on RGB Range Values
        inRange(hsv,Scalar(low_b,low_g,low_r), Scalar(high_b,high_g,high_r),frame_threshold);
        //-- Show the frames
        
	
    imshow("Video Capture",frame);
    imshow("Object Detection",frame_threshold);
    //imshow("circles", frame);
    waitKey(1);
    }
     */
    return 0;
    
 
 
}








